package tools;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import CAEX.CAEXFile;
import CAEX.util.CAEXResourceFactoryImpl;
import filesystem.Folder;
import filesystem.Project;
import filesystem.FilesystemFactory;

public class FileSystemParser {

	static File dir;
	
	static Vector<File> allElements = new Vector<File>();
	static HashMap<File,filesystem.File> fileMap = new HashMap<File, filesystem.File>();
	static HashMap<File,filesystem.Folder> folderMap = new HashMap<File, filesystem.Folder>();
	

	public static void generateModel(String dirAsString, String outputFile) throws IOException{
		
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
	    Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("aml", new CAEXResourceFactoryImpl());
	    m.put("xmi", new XMIResourceFactoryImpl());
	    ResourceSet resSet = new ResourceSetImpl();
		
		
		dir = new File(dirAsString);
		
		if(dir.isDirectory()){
			loadFiles(dir);
		}
		allElements.add(dir);
		
		FilesystemFactory factory = FilesystemFactory.eINSTANCE;
		Project project = factory.createProject();
		
		Iterator<File> allElementsIter = allElements.iterator();
		while(allElementsIter.hasNext()){
			
			File f = allElementsIter.next();
			if(f.isDirectory()){
				Folder folder = factory.createFolder();
				folder.setName(f.getName());
				folder.setAbsoluteName(f.getAbsolutePath());
				folderMap.put(f, folder);
			} else{
				String extension = f.getName().substring(f.getName().lastIndexOf("."), f.getName().length());
				
				//Handle AML files
				if(extension.compareTo(".aml") == 0)
				{
					try {
						CaexPreprocessor.removeAdditionalInformation(f.getAbsolutePath());
						
						Resource res = resSet.getResource(URI.createFileURI(f.getAbsolutePath()),true);
						CAEXFile caexFile = (CAEXFile) res.getContents().get(0).eContents().get(0);
								
						filesystem.AMLFile aml = factory.createAMLFile();
						aml.setAmlFile(caexFile);
						
						aml.setName(f.getName());
						aml.setAbsoluteName(f.getAbsolutePath());
						aml.setExtension(f.getName().substring(f.getName().lastIndexOf("."), f.getName().length()));
						
						fileMap.put(f, aml);
						
						System.out.println(f.getAbsolutePath() + " parsed as AML File");
						
					} catch (Exception e) {
						//TODO
						e.printStackTrace();
					}

				}
				//all other files
				else{
					filesystem.File file = factory.createFile();
					file.setName(f.getName());
					file.setAbsoluteName(f.getAbsolutePath());
					file.setExtension(f.getName().substring(f.getName().lastIndexOf("."), f.getName().length()));
					fileMap.put(f, file);
				}
			}
		}
		
		Iterator<File> allElementsIter2 = allElements.iterator();
		while(allElementsIter2.hasNext()){
			File f = allElementsIter2.next();
			if(f.isDirectory()){
				Folder folder = folderMap.get(f);
				if(f.getAbsolutePath().equals(dir.getAbsolutePath())){
					project.setRoot(folder);
				}else{
					folder.setContainer(folderMap.get(f.getParentFile()));
				}
			}else{
				filesystem.File file = fileMap.get(f);
				if(f.getAbsolutePath().equals(dir.getAbsolutePath())){
					project.setRoot(file);
				}else{
					file.setContainer(folderMap.get(f.getParentFile()));
				}
			}
		}
			
	    Resource resource = resSet.createResource(URI.createURI(outputFile));
	    resource.getContents().add(project);	    

	    try {
	      resource.save(Collections.EMPTY_MAP);
	    } catch (IOException e) {
	      e.printStackTrace();
	    }
	}
	
	private static void loadFiles(File dir) throws FileNotFoundException {		
		File[] files = dir.listFiles();
		if (files != null){
			for (int i=0; i<files.length; i++){
				File file = files[i];
				allElements.add(file);
				if(file.isDirectory()){
					loadFiles(file);
				}
			}
		}
	}

}
